package org.freecompany.brimstone.cache;

import java.io.File;
import java.io.IOException;
import java.net.CacheRequest;
import java.net.CacheResponse;
import java.net.ResponseCache;
import java.net.URI;
import java.net.URLConnection;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import static java.util.logging.Level.FINE;
import static java.util.logging.Logger.getLogger;
import static org.freecompany.brimstone.core.Util.rewrite;

public class ResourceCache extends ResponseCache {

	private static final Logger logger = getLogger( ResponseCache.class.getName());
	
    private File base;
	private ResponseCache chainedCache;

    public ResourceCache() { this( Util.getCacheDirectory()); }
    public ResourceCache( File base) { this( base, null); }
    public ResourceCache( ResponseCache chainedCache) { this( Util.getCacheDirectory(), chainedCache); }
    public ResourceCache( File base, ResponseCache chainedCache) { this.base = base; this.chainedCache = chainedCache; }

    File getCacheLocation( URI uri) {
        if ( !"urn".equalsIgnoreCase( uri.getScheme()) || !uri.getSchemeSpecificPart().toLowerCase().startsWith( "brimstone:")) return null;
		return new File( base, rewrite( uri).replace( '/', File.separatorChar));
    }

    public CacheResponse get( final URI uri, final String requestMethod, final Map< String, List< String>> requestHeaders) throws IOException {
		try {
			return java.security.AccessController.doPrivileged( new java.security.PrivilegedExceptionAction< CacheResponse>() {
					public CacheResponse run() throws IOException {
						logger.log( FINE, "Searching cache for uri ''{0}''.", uri);
						if ( requestMethod != null && !"GET".equals( requestMethod)) return chainedGet( uri, requestMethod, requestHeaders);

						File cacheLocation = getCacheLocation(uri);
						logger.log( FINE, "Using cache location ''{0}''.", cacheLocation);
						if ( cacheLocation == null || !cacheLocation.exists()) return chainedGet( uri, requestMethod, requestHeaders);

						return new ResourceCacheResponse( cacheLocation);
					}
					});
		} catch ( java.security.PrivilegedActionException e) {
			throw ( IOException) e.getException();
		}
    }

    public CacheRequest put( URI uri, URLConnection connection) throws IOException {
        File cacheLocation = getCacheLocation( uri);
        if ( cacheLocation == null) return chainedPut( uri, connection);

		return new ResourceCacheRequest(cacheLocation, connection);
    }

    private CacheResponse chainedGet( URI uri, String requestMethod, Map< String, List< String>> requestHeaders) throws IOException {
		if ( chainedCache == null) return null;
		return chainedCache.get( uri, requestMethod, requestHeaders);
	}

    private CacheRequest chainedPut( URI uri, URLConnection connection) throws IOException {
		if ( chainedCache == null) return null;
		return chainedCache.put( uri, connection);
	}

    static File headersFile( File file) {
        return new File( file.getParentFile(), basename( file) + ".headers");
    }

	private static String basename( File file) {
        String name = file.getName();
        int pos = name.lastIndexOf( '.');
        if ( pos >= 0) name = name.substring( 0, pos);
		return name;
	}
}
