package org.freecompany.brimstone.cache;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URI;
import java.net.ResponseCache;
import java.util.Properties;
import java.util.logging.Logger;

import static java.util.logging.Level.SEVERE;
import static java.util.logging.Logger.getLogger;

public class Util {

	public static final String CACHE_DIR = "brimstone.cache.dir";

	private static final Logger logger = getLogger( Util.class.getName());
	private static final File cache;

	static {
		cache = java.security.AccessController.doPrivileged( new java.security.PrivilegedAction< File>() {
				public File run() {
					String def = System.getProperty( "user.home") + File.separator + ".brimstone" + File.separator + "cache";
					return new File( System.getProperty( CACHE_DIR, def));
				}
				});
	}

	public static File getCacheDirectory() {
		return cache;
	}

	public static File loadToCache( URI uri) throws IOException {
		ResponseCache cache = ResponseCache.getDefault();
		if ( cache == null || !( cache instanceof ResourceCache)) {
			cache = new ResourceCache();
		} else {
			uri.toURL().openConnection().getInputStream().close();
		}
		File result = (( ResourceCache) cache).getCacheLocation( uri);
		if ( !result.exists()) {
			logger.log( SEVERE, "The resources could not be located in the cache at ''{0}''.", result);
			throw new IOException( "An error occured caching the requested resource.");
		}
		return result;
	}
}
